@@ -104,6 +104,32 @@
 #'  of variance.
 #' @param heterogeneity_var Variable name as a character string to use for 
 #'  heterogeneity of variance simulation.
+#' @param cross_class_params A list of named parameters when cross classified 
+#'  data structures are desired. Must include the following arguments:
+#'   \itemize{
+#'    \item num_ids: The number of cross classified clusters. These are in 
+#'         addition to the typical cluster ids
+#'    \item random_param: This argument is a list of arguments passed to 
+#'       \code{\link{sim_rand_eff}}. These must include:
+#'      \itemize{
+#'       \item random_var: The variance of the cross classified random effect
+#'       \item rand_gen: The random generating function used to generate the 
+#'          cross classified random effect.
+#'      }
+#'      Optional elements are:
+#'    \itemize{
+#'        \item ther: Theorectial mean and variance from rand_gen,
+#'        \item ther_sim: Simulate mean/variance for standardization purposes,
+#'        \item cor_vars: Correlation between random effects,
+#'        \item ...: Additional parameters needed for rand_gen function.
+#'    } 
+#'   }
+#' @param knot_args A nested list of named knot arguments. See \code{\link{sim_knot}} 
+#'  for more details. Arguments must include:
+#'    \itemize{
+#'      \item var
+#'      \item knot_locations
+#'    }
 #' @param ... Not currently used.
 #' @import stats
 #' @export 
@@ -187,24 +213,28 @@ sim_reg <- function(fixed, random, random3, fixed_param,
                     unbal_design = list("level2" = NULL, "level3" = NULL), 
                     lvl1_err_params = NULL, arima_mod = list(NULL), 
                     contrasts = NULL, homogeneity = TRUE,
-                    heterogeneity_var = NULL, ...) {
+                    heterogeneity_var = NULL, cross_class_params = NULL, 
+                    knot_args = list(NULL), ...) {
   
   if(data_str == "single"){
     sim_reg_single(fixed, fixed_param, cov_param, n, error_var, with_err_gen, 
                    arima, data_str, cor_vars, fact_vars, lvl1_err_params, 
-                   arima_mod, contrasts, homogeneity, heterogeneity_var, ...)
+                   arima_mod, contrasts, homogeneity, heterogeneity_var, 
+                   knot_args, ...)
   } else {
   	if (is.null(k)){
   	  sim_reg_nested(fixed, random, fixed_param, random_param, cov_param, n, p, 
   	                 error_var, with_err_gen, arima, data_str, cor_vars, 
   	                 fact_vars, unbal, unbal_design, lvl1_err_params, 
-  	                 arima_mod, contrasts, homogeneity, heterogeneity_var, ...)
+  	                 arima_mod, contrasts, homogeneity, heterogeneity_var, 
+  	                 cross_class_params, knot_args, ...)
   } else {
     sim_reg_nested3(fixed, random, random3, fixed_param, random_param, 
                     random_param3, cov_param, k, n, p, error_var, with_err_gen, 
                     arima, data_str, cor_vars, fact_vars, unbal, 
                     unbal_design, lvl1_err_params, arima_mod, contrasts,
-                    homogeneity, heterogeneity_var, ...)
+                    homogeneity, heterogeneity_var, 
+                    cross_class_params, knot_args, ...)
   }
  }
 }
@@ -302,6 +332,32 @@ sim_reg <- function(fixed, random, random3, fixed_param,
 #' @param outcome_type A vector specifying the type of outcome, must be either
 #'   logistic or poisson. Logitstic outcome will be 0/1 and poisson outcome will
 #'   be counts.
+#' @param cross_class_params A list of named parameters when cross classified 
+#'  data structures are desired. Must include the following arguments:
+#'   \itemize{
+#'    \item num_ids: The number of cross classified clusters. These are in 
+#'         addition to the typical cluster ids
+#'    \item random_param: This argument is a list of arguments passed to 
+#'       \code{\link{sim_rand_eff}}. These must include:
+#'      \itemize{
+#'       \item random_var: The variance of the cross classified random effect
+#'       \item rand_gen: The random generating function used to generate the 
+#'          cross classified random effect.
+#'      }
+#'      Optional elements are:
+#'    \itemize{
+#'        \item ther: Theorectial mean and variance from rand_gen,
+#'        \item ther_sim: Simulate mean/variance for standardization purposes,
+#'        \item cor_vars: Correlation between random effects,
+#'        \item ...: Additional parameters needed for rand_gen function.
+#'    } 
+#'   }
+#' @param knot_args A nested list of named knot arguments. See \code{\link{sim_knot}} 
+#'  for more details. Arguments must include:
+#'    \itemize{
+#'      \item var
+#'      \item knot_locations
+#'    }
 #' @param ... Not currently used.
 #' @export 
 #' 
@@ -373,21 +429,24 @@ sim_glm <- function(fixed, random, random3, fixed_param, random_param = list(),
                     data_str, cor_vars = NULL, fact_vars = list(NULL), 
                     unbal = list("level2" = FALSE, "level3" = FALSE), 
                     unbal_design = list("level2" = NULL, "level3" = NULL),
-                    contrasts = NULL, outcome_type, ...) {
+                    contrasts = NULL, outcome_type, 
+                    cross_class_params = NULL, knot_args = list(NULL), ...) {
   
   if(data_str == "single"){
     sim_glm_single(fixed, fixed_param, cov_param, n, data_str, 
-                   cor_vars, fact_vars, contrasts, outcome_type, ...)
+                   cor_vars, fact_vars, contrasts, outcome_type, 
+                   knot_args, ...)
   } else {
     if (is.null(k)){
       sim_glm_nested(fixed, random, fixed_param, random_param, cov_param, n, p, 
                      data_str, cor_vars, fact_vars, unbal, unbal_design, 
-                     contrasts, outcome_type, ...)
+                     contrasts, outcome_type, cross_class_params, 
+                     knot_args, ...)
     } else {
       sim_glm_nested3(fixed, random, random3, fixed_param, random_param, 
                       random_param3, cov_param, k, n, p, data_str, cor_vars, 
                       fact_vars, unbal, unbal_design, contrasts,
-                      outcome_type, ...)
+                      outcome_type, cross_class_params, knot_args, ...)
     }
   }
 }
