@@ -3,8 +3,10 @@ context('data_glm')
 test_that('probabilities between 0 and 1', {
   Xmat <- cbind(rnorm(50), rnorm(50), rnorm(50))
   beta <- c(4, 3, 1)
-  expect_gte(min(data_glm_single(Xmat, beta, 50)$logistic), 0)
-  expect_lte(max(data_glm_single(Xmat, beta, 50)$logistic), 1)
+  expect_gte(min(data_glm_single(Xmat, beta, 50, 
+                                 outcome_type = 'logistic')$logistic), 0)
+  expect_lte(max(data_glm_single(Xmat, beta, 50, 
+                                 outcome_type = 'logistic')$logistic), 1)
   
   # z_mat <- Xmat[, 1]
   # rand_eff <- c(rnorm(50))
@@ -14,13 +16,13 @@ test_that('probabilities between 0 and 1', {
 test_that('correct length', {
   Xmat <- cbind(rnorm(50), rnorm(50), rnorm(50))
   beta <- c(4, 3, 1)
-  expect_equal(nrow(data_glm_single(Xmat, beta, 50)), 50)
+  expect_equal(nrow(data_glm_single(Xmat, beta, 50, outcome_type = 'logistic')), 50)
 })
 
 test_that('sim_data are 0 or 1s inclusive', {
   Xmat <- cbind(rnorm(50), rnorm(50), rnorm(50))
   beta <- c(4, 3, 1)
-  expect_length(table(data_glm_single(Xmat, beta, 50)$sim_data), 2)
+  expect_length(table(data_glm_single(Xmat, beta, 50, outcome_type = 'logistic')$sim_data), 2)
 })
 
 context('data_reg')
